/*

Copyright (c) 2008 – 2014 Texas Instruments Incorporated

All rights reserved not granted herein.

Limited License.  

Texas Instruments Incorporated grants a world-wide, royalty-free, non-exclusive license under copyrights and patents it now or hereafter owns or controls to make, have made, use, import, offer to sell and sell ("Utilize") this software subject to the terms herein.  With respect to the foregoing patent license, such license is granted  solely to the extent that any such patent is necessary to Utilize the software alone.  The patent license shall not apply to any combinations which include this software, other than combinations with devices manufactured by or for TI (“TI Devices”).  No hardware patent is licensed hereunder.

Redistributions must preserve existing copyright notices and reproduce this license (including the above copyright notice and the disclaimer and (if applicable) source code license limitations below) in the documentation and/or other materials provided with the distribution

Redistribution and use in binary form, without modification, are permitted provided that the following conditions are met:

*	No reverse engineering, decompilation, or disassembly of this software is permitted with respect to any software provided in binary form.

*	any redistribution and use are licensed by TI for use only with TI Devices.

*	Nothing shall obligate TI to provide you with source code for the software licensed and provided to you in object code.

If software source code is provided to you, modification and redistribution of the source code are permitted provided that the following conditions are met:

*	any redistribution and use of the source code, including any resulting derivative works, are licensed by TI for use only with TI Devices.

*	any redistribution and use of any object code compiled from the source code and any resulting derivative works, are licensed by TI for use only with TI Devices.

Neither the name of Texas Instruments Incorporated nor the names of its suppliers may be used to endorse or promote products derived from this software without specific prior written permission.

DISCLAIMER.

THIS SOFTWARE IS PROVIDED BY TI AND TI’S LICENSORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL TI AND TI’S LICENSORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

/**
 * \file BSL432_Peripheral_Interface_eUSCI_UART_SPI_I2C.c
 * \brief Source file for the BSL Peripheral Interface functions.
 */

#include "BSL432_API.h"
#include "BSL432_Peripheral_Interface.h"
#include "BSL432_Command_Interpreter.h"
#include "BSL432_Command_Definitions.h"
#include "BSL432_device_file.h"

#pragma RETAIN(BSL432_VersionPeripheralInterface)
#pragma DATA_SECTION(BSL432_VersionPeripheralInterface, ".BSL432_VERSION_PI")
const uint8_t BSL432_VersionPeripheralInterface[2] = {0x01, 0x02};

BSL432_eventFlags BSL432_PI_eventFlags = Flags_rxPacketOngoing;

#pragma DATA_ALIGN(BSL432_RAM_TX_Buf, 2);
uint8_t BSL432_RAM_TX_Buf[BSL432_MAX_BUFFER_SIZE + 6];
#pragma DATA_ALIGN(BSL432_RAM_RX_Buf, 2);
uint8_t BSL432_RAM_RX_Buf[BSL432_MAX_BUFFER_SIZE + 6];

/* Function pointers to the current peripherals functions */
void (*BSL432_PI_sendByte)(uint8_t) = 0;
uint8_t (*BSL432_PI_receiveByte)() = 0;
void (*BSL432_PI_sendData)(uint32_t) = 0;
BSL432_eventFlags (*BSL432_PI_receivePacket)() = 0;

BSL432_ActivePeripherals BSL432_ActivePeripheral = BSL432_AP_Auto;
uint32_t BSL432_UART_MODULE = EUSCI_A0_MODULE;
uint32_t BSL432_SPI_MODULE = EUSCI_B0_MODULE;
uint32_t BSL432_I2C_MODULE = EUSCI_B2_MODULE;
uint16_t BSL432_I2C_SLAVE_ADDRESS = 0x48;
/* UART pin settings for configuration after baud rate detection */
uint8_t BSL432_UART_PORT = 0;
uint16_t BSL432_UART_RXD_PIN = 0;
uint16_t BSL432_UART_TXD_PIN = 0;
uint8_t BSL432_UART_RXD_MODE = 0;
uint8_t BSL432_UART_TXD_MODE = 0;

uint32_t BSL432_PI_RxDataIndexI2C = 0;
uint32_t BSL432_PI_TxDataIndexI2C = 0;

/**
 * PI initialization routine configures all peripherals.
 *
 * \return None
 */
void BSL432_PI_init()
{
	// Configure clock system
	// MCLK = 12 MHz, ACLK = 32768 Hz
	MAP_CS_setReferenceOscillatorFrequency(CS_REFO_32KHZ);
	MAP_CS_setDCOCenteredFrequency(CS_DCO_FREQUENCY_12);
	MAP_CS_initClockSignal(CS_MCLK, CS_DCOCLK_SELECT, CS_CLOCK_DIVIDER_1);
	MAP_CS_initClockSignal(CS_ACLK, CS_REFO_32KHZ, CS_CLOCK_DIVIDER_1);

	// Configure power mode and core voltage
	MAP_PCM_setPowerState(PCM_AM_LDO_VCORE0);

	// Extract BSL parameters given by boot code or user application
	// Interface selection (15:13)
	if(((BSL432_BSL_PARAMS >> 13) & 0b111) == BSL432_AP_UART)
	{
		BSL432_ActivePeripheral = BSL432_AP_UART;
	}
	else if(((BSL432_BSL_PARAMS >> 13) & 0b111) == BSL432_AP_SPI)
	{
		BSL432_ActivePeripheral = BSL432_AP_SPI;
	}
	else if(((BSL432_BSL_PARAMS >> 13) & 0b111) == BSL432_AP_I2C)
	{
		BSL432_ActivePeripheral = BSL432_AP_I2C;
	}
	else  // Default to auto interface selection
	{
		BSL432_ActivePeripheral = BSL432_AP_Auto;
	}

	// I2C slave address from BSL parameters (25:16)
	BSL432_I2C_SLAVE_ADDRESS = (BSL432_BSL_PARAMS >> 16) & 0x7F;  // Note: DriverLib supports only 7-bit slave address

#ifdef _DEBUG
#warning Debug: Using peripheral override
	BSL432_ActivePeripheral = BSL432_AP_Auto;  // Peripheral override for debug
#endif

	BSL432_PI_configureInterfaces();

	// Initialize peripheral
	switch(BSL432_ActivePeripheral) {
		case BSL432_AP_Auto:
			BSL432_PI_initUARTdetect();
			BSL432_PI_initSPI();
			BSL432_PI_initI2C();
			break;
		case BSL432_AP_UART:
			BSL432_PI_initUARTdetect();
			break;
		case BSL432_AP_SPI:
			BSL432_PI_initSPI();
			break;
		case BSL432_AP_I2C:
			BSL432_PI_initI2C();
			break;
		default:
			break;
	}

	BSL432_ReceiveBuffer = BSL432_RAM_RX_Buf;
	BSL432_SendBuffer = BSL432_RAM_TX_Buf;
	// I2C uses a sligt offset with the send buffer, which is set after communication on I2C was detected.
}

/**
 * PI interface configuration routine that sets up the used peripheral modules and handles the port mapping.
 *
 * \return None
 *
 */
void BSL432_PI_configureInterfaces()
{
	// UART module
	if(BSL432_ActivePeripheral == BSL432_AP_Auto || BSL432_ActivePeripheral == BSL432_AP_UART)
	{
		if(BSL432_BSL_PER_IF_SEL_UART_MOD == 0x0)  // eUSCI_A
		{
			if(BSL432_BSL_PER_IF_SEL_UART_INST == 0x0)
			{
				BSL432_UART_MODULE = EUSCI_A0_MODULE;
			}
			else if(BSL432_BSL_PER_IF_SEL_UART_INST == 0x1)
			{
				BSL432_UART_MODULE = EUSCI_A1_MODULE;
			}
			else if(BSL432_BSL_PER_IF_SEL_UART_INST == 0x2)
			{
				BSL432_UART_MODULE = EUSCI_A2_MODULE;
			}
			else if(BSL432_BSL_PER_IF_SEL_UART_INST == 0x3)
			{
				BSL432_UART_MODULE = EUSCI_A3_MODULE;
			}
		}
	}

	// SPI module
	if(BSL432_ActivePeripheral == BSL432_AP_Auto || BSL432_ActivePeripheral == BSL432_AP_SPI)
	{
		if(BSL432_BSL_PER_IF_SEL_SPI_MOD == 0x0)  // eUSCI_A
		{
			if(BSL432_BSL_PER_IF_SEL_SPI_INST == 0x0)
			{
				BSL432_SPI_MODULE = EUSCI_A0_MODULE;
			}
			else if(BSL432_BSL_PER_IF_SEL_SPI_INST == 0x1)
			{
				BSL432_SPI_MODULE = EUSCI_A1_MODULE;
			}
			else if(BSL432_BSL_PER_IF_SEL_SPI_INST == 0x2)
			{
				BSL432_SPI_MODULE = EUSCI_A2_MODULE;
			}
			else if(BSL432_BSL_PER_IF_SEL_SPI_INST == 0x3)
			{
				BSL432_SPI_MODULE = EUSCI_A3_MODULE;
			}
		}
		else if(BSL432_BSL_PER_IF_SEL_SPI_MOD == 0x1)  // eUSCI_B
		{
			if(BSL432_BSL_PER_IF_SEL_SPI_INST == 0x0)
			{
				BSL432_SPI_MODULE = EUSCI_B0_MODULE;
			}
			else if(BSL432_BSL_PER_IF_SEL_SPI_INST == 0x1)
			{
				BSL432_SPI_MODULE = EUSCI_B1_MODULE;
			}
			else if(BSL432_BSL_PER_IF_SEL_SPI_INST == 0x2)
			{
				BSL432_SPI_MODULE = EUSCI_B2_MODULE;
			}
			else if(BSL432_BSL_PER_IF_SEL_SPI_INST == 0x3)
			{
				BSL432_SPI_MODULE = EUSCI_B3_MODULE;
			}
		}
	}

	// I2C module
	if(BSL432_ActivePeripheral == BSL432_AP_Auto || BSL432_ActivePeripheral == BSL432_AP_I2C)
	{
		if(BSL432_BSL_PER_IF_SEL_I2C_MOD == 0x0)  // eUSCI_B
		{
			if(BSL432_BSL_PER_IF_SEL_I2C_INST == 0x0)
			{
				BSL432_I2C_MODULE = EUSCI_B0_MODULE;
			}
			else if(BSL432_BSL_PER_IF_SEL_I2C_INST == 0x1)
			{
				BSL432_I2C_MODULE = EUSCI_B1_MODULE;
			}
			else if(BSL432_BSL_PER_IF_SEL_I2C_INST == 0x2)
			{
				BSL432_I2C_MODULE = EUSCI_B2_MODULE;
			}
			else if(BSL432_BSL_PER_IF_SEL_I2C_INST == 0x3)
			{
				BSL432_I2C_MODULE = EUSCI_B3_MODULE;
			}
		}
	}

	const uint8_t ports_mapping[] =  // Port mapping between BSL configuration and Driver Library
	{
		GPIO_PORT_P1, GPIO_PORT_P2, GPIO_PORT_P3, GPIO_PORT_P4,
		GPIO_PORT_P5, GPIO_PORT_P6, GPIO_PORT_P7, GPIO_PORT_P8,
		GPIO_PORT_P9, GPIO_PORT_P10, GPIO_PORT_PJ, GPIO_PORT_P1,  // Note: only 11 ports supported in DriverLib for now
		GPIO_PORT_P1, GPIO_PORT_P1, GPIO_PORT_P1, GPIO_PORT_P1
	};

	const uint16_t pins_mapping[] =  // Pin mapping between BSL configuration and Driver Library
	{
		GPIO_PIN0, GPIO_PIN1, GPIO_PIN2, GPIO_PIN3,
		GPIO_PIN4, GPIO_PIN5, GPIO_PIN6, GPIO_PIN7
	};

	// PxSELy mapping between BSL configuration and Driver Library: mode_mapping[PxSEL1][PxSEL0]
	const uint8_t mode_mapping[2][2] =
	{
		{0, GPIO_PRIMARY_MODULE_FUNCTION},
		{GPIO_SECONDARY_MODULE_FUNCTION, GPIO_TERTIARY_MODULE_FUNCTION}
	};

	// UART module muxing
	if(BSL432_ActivePeripheral == BSL432_AP_Auto || BSL432_ActivePeripheral == BSL432_AP_UART)
	{
		if(BSL432_BSL_PER_IF_SEL_UART_MUX == 0x1)
		{
			// Save UART configuration for now, configure later in detection and active communication routines
			BSL432_UART_PORT = ports_mapping[BSL432_BSL_PORTCNF_UART_UART_PORT];
			BSL432_UART_RXD_PIN = pins_mapping[BSL432_BSL_PORTCNF_UART_BITPOS_RXD];
			BSL432_UART_TXD_PIN = pins_mapping[BSL432_BSL_PORTCNF_UART_BITPOS_TXD];
			BSL432_UART_RXD_MODE = mode_mapping[BSL432_BSL_PORTCNF_UART_PSEL1_RXD][BSL432_BSL_PORTCNF_UART_PSEL0_RXD];
			BSL432_UART_TXD_MODE = mode_mapping[BSL432_BSL_PORTCNF_UART_PSEL1_TXD][BSL432_BSL_PORTCNF_UART_PSEL0_TXD];
		}
	}

	// SPI module muxing
	if(BSL432_ActivePeripheral == BSL432_AP_Auto || BSL432_ActivePeripheral == BSL432_AP_SPI)
	{
		if(BSL432_BSL_PER_IF_SEL_SPI_MUX == 0x1)
		{
			if(BSL432_BSL_PORTCNF_SPI_PSEL0_SOMI != 0 || BSL432_BSL_PORTCNF_SPI_PSEL1_SOMI != 0)
			{
				MAP_GPIO_setAsPeripheralModuleFunctionInputPin(ports_mapping[BSL432_BSL_PORTCNF_SPI_SPI_PORT],
						pins_mapping[BSL432_BSL_PORTCNF_SPI_BITPOS_SOMI],
						mode_mapping[BSL432_BSL_PORTCNF_SPI_PSEL1_SOMI][BSL432_BSL_PORTCNF_SPI_PSEL0_SOMI]);
			}
			if(BSL432_BSL_PORTCNF_SPI_PSEL0_SIMO != 0 || BSL432_BSL_PORTCNF_SPI_PSEL1_SIMO != 0)
			{
				MAP_GPIO_setAsPeripheralModuleFunctionInputPin(ports_mapping[BSL432_BSL_PORTCNF_SPI_SPI_PORT],
						pins_mapping[BSL432_BSL_PORTCNF_SPI_BITPOS_SIMO],
						mode_mapping[BSL432_BSL_PORTCNF_SPI_PSEL1_SIMO][BSL432_BSL_PORTCNF_SPI_PSEL0_SIMO]);
			}
			if(BSL432_BSL_PORTCNF_SPI_PSEL0_CLK != 0 || BSL432_BSL_PORTCNF_SPI_PSEL1_CLK != 0)
			{
				MAP_GPIO_setAsPeripheralModuleFunctionInputPin(ports_mapping[BSL432_BSL_PORTCNF_SPI_SPI_PORT],
						pins_mapping[BSL432_BSL_PORTCNF_SPI_BITPOS_CLK],
						mode_mapping[BSL432_BSL_PORTCNF_SPI_PSEL1_CLK][BSL432_BSL_PORTCNF_SPI_PSEL0_CLK]);
			}
			if(BSL432_BSL_PORTCNF_SPI_PSEL0_STE != 0 || BSL432_BSL_PORTCNF_SPI_PSEL1_STE != 0)
			{
				MAP_GPIO_setAsPeripheralModuleFunctionInputPin(ports_mapping[BSL432_BSL_PORTCNF_SPI_SPI_PORT],
						pins_mapping[BSL432_BSL_PORTCNF_SPI_BITPOS_STE],
						mode_mapping[BSL432_BSL_PORTCNF_SPI_PSEL1_STE][BSL432_BSL_PORTCNF_SPI_PSEL0_STE]);
			}
		}
	}

	// I2C module muxing
	if(BSL432_ActivePeripheral == BSL432_AP_Auto || BSL432_ActivePeripheral == BSL432_AP_I2C)
	{
		if(BSL432_BSL_PER_IF_SEL_I2C_MUX == 0x1)
		{
			if(BSL432_BSL_PORTCNF_I2C_PSEL0_SCL != 0 || BSL432_BSL_PORTCNF_I2C_PSEL1_SCL != 0)
			{
				MAP_GPIO_setAsPeripheralModuleFunctionInputPin(ports_mapping[BSL432_BSL_PORTCNF_I2C_I2C_PORT],
						pins_mapping[BSL432_BSL_PORTCNF_I2C_BITPOS_SCL],
						mode_mapping[BSL432_BSL_PORTCNF_I2C_PSEL1_SCL][BSL432_BSL_PORTCNF_I2C_PSEL0_SCL]);
			}
			if(BSL432_BSL_PORTCNF_I2C_PSEL0_SDA != 0 || BSL432_BSL_PORTCNF_I2C_PSEL1_SDA != 0)
			{
				MAP_GPIO_setAsPeripheralModuleFunctionInputPin(ports_mapping[BSL432_BSL_PORTCNF_I2C_I2C_PORT],
						pins_mapping[BSL432_BSL_PORTCNF_I2C_BITPOS_SDA],
						mode_mapping[BSL432_BSL_PORTCNF_I2C_PSEL1_SDA][BSL432_BSL_PORTCNF_I2C_PSEL0_SDA]);
			}
		}
	}
}

/**
 * PI routine that scans all active interfaces for received data
 *
 * \return None
 */
void BSL432_PI_detectInterface()
{
	// BSL timeout loop
	while(1)
	{
		// Check if timeout period passed
		if(MAP_Timer_A_getInterruptStatus(TIMER_A0_MODULE) == TIMER_A_INTERRUPT_PENDING)
		{
#ifdef _DEBUG
			MAP_GPIO_setOutputLowOnPin(GPIO_PORT_P1, GPIO_PIN0);
			while(1);
#endif
			BSL432_API_enterLPM();
		}

		// Check for activity on the peripheral interfaces
		if(BSL432_ActivePeripheral == BSL432_AP_Auto || BSL432_ActivePeripheral == BSL432_AP_UART)
		{
			if(!(BSL432_PI_UARTdetect() == BSL432_UARTBaudRatesAuto))
			{
				BSL432_ActivePeripheral = BSL432_AP_UART;
				break;
			}
		}
		if(BSL432_ActivePeripheral == BSL432_AP_Auto || BSL432_ActivePeripheral == BSL432_AP_SPI)
		{
			if(SPI_getInterruptStatus(BSL432_SPI_MODULE) & EUSCI_B_SPI_RECEIVE_INTERRUPT)
			{
				BSL432_ActivePeripheral = BSL432_AP_SPI;
				/* Configure function pointers for current peripheral */
				BSL432_PI_sendByte = &BSL432_PI_sendByteSPI;
				BSL432_PI_receiveByte = &BSL432_PI_receiveByteSPI;
				BSL432_PI_sendData = &BSL432_PI_sendDataSPI;
				BSL432_PI_receivePacket = BSL432_PI_receivePacketSPI;
				break;
			}
		}
		if(BSL432_ActivePeripheral == BSL432_AP_Auto || BSL432_ActivePeripheral == BSL432_AP_I2C)
		{
			if(I2C_getInterruptStatus(BSL432_I2C_MODULE, EUSCI_B_I2C_RECEIVE_INTERRUPT0))
			{
				BSL432_ActivePeripheral = BSL432_AP_I2C;
				/* Configure function pointers for current peripheral */
				BSL432_PI_sendByte = &BSL432_PI_sendByteI2C;
				BSL432_PI_receiveByte = 0;
				BSL432_PI_sendData = &BSL432_PI_sendDataI2C;
				BSL432_PI_receivePacket = &BSL432_PI_receivePacketI2C;
				BSL432_SendBuffer = &BSL432_RAM_TX_Buf[4]; // Offset send buffer for I2C
				break;
			}
		}
	}
}

/**
 * PI initialization routine for passive (RX only) UART auto baudrate detection.
 *
 * \return None
 */
void BSL432_PI_initUARTdetect()
{
	/* Configure and enable SysTick. Set the reload value to the maximum */
	MAP_SysTick_setPeriod(1000000);
	MAP_SysTick_enableModule();

	/* Set up RX pin for input with pull-up for baud rate detection. */
	MAP_GPIO_setAsInputPinWithPullUpResistor(BSL432_UART_PORT, BSL432_UART_RXD_PIN);
}

/**
 * PI auto baud rate detection
 * Checks for the used baud rate with the help of the 0xFF synchronisation character.
 *
 * /returns BSL432_UARTBaudRates of the detected baud rate, or auto if none.
 */
BSL432_UARTBaudRates BSL432_PI_UARTdetect()
{
	/*
	 * Synchronization charater 0xFF
	 * ____   ________________   ____
	 *     |_|1 2 3 4 5 6 7 8 |_|
	 */

	/* Check if pin is already low... */
	SCS_STRVR = BSL432_UART_BAUDRATE_CPS_9600_MAX;  // Timeout of lowest baud rate
	SCS_STCVR = 0; // Reset SysTick to period value
	SCS_STCSR &= ~(SCS_STCSR_COUNTFLAG);
	if(!(GPIO_getInputPinValue(BSL432_UART_PORT, BSL432_UART_RXD_PIN)))
	{
		/* ... wait for pin high with */
		while(!(GPIO_getInputPinValue(BSL432_UART_PORT, BSL432_UART_RXD_PIN)))
		{
			/* Check for timeout */
			if(SCS_STCSR & SCS_STCSR_COUNTFLAG)
			{
				return BSL432_UARTBaudRatesAuto;
			}
		}
		/* Pin went high, count for 8 symbols until pin goes low. */
		SCS_STRVR = (BSL432_UART_BAUDRATE_CPS_9600_MAX * 8);  // Timeout for lowest baud rate * 8 symbols
		SCS_STCVR = 0; // Reset SysTick to period value
		SCS_STCSR &= ~(SCS_STCSR_COUNTFLAG);
		while(GPIO_getInputPinValue(BSL432_UART_PORT, BSL432_UART_RXD_PIN))
		{
			/* Check for timeout */
			if(SCS_STCSR & SCS_STCSR_COUNTFLAG)
			{
				return BSL432_UARTBaudRatesAuto;
			}
		}
		/* Evaluate elapsed time */
		uint32_t currentSysTick = SCS_STCVR;
		currentSysTick = (BSL432_UART_BAUDRATE_CPS_9600_MAX * 8) - currentSysTick;
		if(currentSysTick > (8 * BSL432_UART_BAUDRATE_CPS_115200_MIN) && currentSysTick < (8 * BSL432_UART_BAUDRATE_CPS_115200_MAX))  // 115200 baud
		{
			BSL432_PI_initUART(BSL432_UARTBaudRates115200);
			BSL432_PI_sendByteUART(BSL432_ACK);
			return BSL432_UARTBaudRates115200;
		}
		else if(currentSysTick > (8 * BSL432_UART_BAUDRATE_CPS_57600_MIN) && currentSysTick < (8 * BSL432_UART_BAUDRATE_CPS_57600_MAX))  // 57600 baud
		{
			BSL432_PI_initUART(BSL432_UARTBaudRates57600);
			BSL432_PI_sendByteUART(BSL432_ACK);
			return BSL432_UARTBaudRates57600;
		}
		else if(currentSysTick > (8 * BSL432_UART_BAUDRATE_CPS_38400_MIN) && currentSysTick < (8 * BSL432_UART_BAUDRATE_CPS_38400_MAX))  // 38400 baud
		{
			BSL432_PI_initUART(BSL432_UARTBaudRates38400);
			BSL432_PI_sendByteUART(BSL432_ACK);
			return BSL432_UARTBaudRates38400;
		}
		else if(currentSysTick > (8 * BSL432_UART_BAUDRATE_CPS_19200_MIN) && currentSysTick < (8 * BSL432_UART_BAUDRATE_CPS_19200_MAX))  // 19200 baud
		{
			BSL432_PI_initUART(BSL432_UARTBaudRates19200);
			BSL432_PI_sendByteUART(BSL432_ACK);
			return BSL432_UARTBaudRates19200;
		}
		else if(currentSysTick > (8 * BSL432_UART_BAUDRATE_CPS_9600_MIN) && currentSysTick < (8 * BSL432_UART_BAUDRATE_CPS_9600_MAX))  // 9600 baud
		{
			BSL432_PI_initUART(BSL432_UARTBaudRates9600);
			BSL432_PI_sendByteUART(BSL432_ACK);
			return BSL432_UARTBaudRates9600;
		}
		else
		{
			/* No known baud rate detected */
			return BSL432_UARTBaudRatesAuto;
		}
	}
	return BSL432_UARTBaudRatesAuto;
}

/**
 * PI initialization routine for UART.
 *
 * \param BSL432_UARTBaudRate The baud rate parameter.
 *
 * \return None
 */
void BSL432_PI_initUART(BSL432_UARTBaudRates BSL432_UARTBaudRate)
{
	const UART_Config BSL432_UARTConfig_auto =
	{
		EUSCI_A_UART_CLOCKSOURCE_SMCLK,
		78, // UCBRx
		2,  // firstModReg UCBRFx
		0,  // secondMod UCBRSx
		EUSCI_A_UART_EVEN_PARITY,
		EUSCI_A_UART_LSB_FIRST,
		EUSCI_A_UART_ONE_STOP_BIT,
		EUSCI_A_UART_AUTOMATIC_BAUDRATE_DETECTION_MODE,
		EUSCI_A_UART_OVERSAMPLING_BAUDRATE_GENERATION
	};

	const UART_Config BSL432_UARTConfig_9600 =
	{
		EUSCI_A_UART_CLOCKSOURCE_SMCLK,
		78,  // UCBRx
		2,   // firstModReg UCBRFx
		0,   // secondMod UCBRSx
		EUSCI_A_UART_EVEN_PARITY,
		EUSCI_A_UART_LSB_FIRST,
		EUSCI_A_UART_ONE_STOP_BIT,
		EUSCI_A_UART_MODE,
		EUSCI_A_UART_OVERSAMPLING_BAUDRATE_GENERATION
	};

	const UART_Config BSL432_UARTConfig_19200 =
	{
		EUSCI_A_UART_CLOCKSOURCE_SMCLK,
		39,  // UCBRx
		1,   // firstModReg UCBRFx
		0,   // secondMod UCBRSx
		EUSCI_A_UART_EVEN_PARITY,
		EUSCI_A_UART_LSB_FIRST,
		EUSCI_A_UART_ONE_STOP_BIT,
		EUSCI_A_UART_MODE,
		EUSCI_A_UART_OVERSAMPLING_BAUDRATE_GENERATION
	};

	const UART_Config BSL432_UARTConfig_38400 =
	{
		EUSCI_A_UART_CLOCKSOURCE_SMCLK,
		19,    // UCBRx
		8,     // firstModReg UCBRFx
		0x65,  // secondMod UCBRSx
		EUSCI_A_UART_EVEN_PARITY,
		EUSCI_A_UART_LSB_FIRST,
		EUSCI_A_UART_ONE_STOP_BIT,
		EUSCI_A_UART_MODE,
		EUSCI_A_UART_OVERSAMPLING_BAUDRATE_GENERATION
	};

	const UART_Config BSL432_UARTConfig_57600 =
	{
		EUSCI_A_UART_CLOCKSOURCE_SMCLK,
		13,    // UCBRx
		0,     // firstModReg UCBRFx
		0x25,  // secondMod UCBRSx
		EUSCI_A_UART_EVEN_PARITY,
		EUSCI_A_UART_LSB_FIRST,
		EUSCI_A_UART_ONE_STOP_BIT,
		EUSCI_A_UART_MODE,
		EUSCI_A_UART_OVERSAMPLING_BAUDRATE_GENERATION
	};

	const UART_Config BSL432_UARTConfig_115200 =
	{
		EUSCI_A_UART_CLOCKSOURCE_SMCLK,
		6,     // UCBRx
		8,     // firstModReg UCBRFx
		0x20,  // secondMod UCBRSx
		EUSCI_A_UART_EVEN_PARITY,
		EUSCI_A_UART_LSB_FIRST,
		EUSCI_A_UART_ONE_STOP_BIT,
		EUSCI_A_UART_MODE,
		EUSCI_A_UART_OVERSAMPLING_BAUDRATE_GENERATION
	};

	/* Disable SysTick */
	MAP_SysTick_disableModule();

	/* Selecting RXD and TXD pins in UART mode */
	if(BSL432_UART_RXD_MODE != 0)
	{
		MAP_GPIO_setAsPeripheralModuleFunctionInputPin(BSL432_UART_PORT,
				BSL432_UART_RXD_PIN, BSL432_UART_RXD_MODE);
	}
	if(BSL432_UART_TXD_MODE != 0)
	{
		MAP_GPIO_setAsPeripheralModuleFunctionInputPin(BSL432_UART_PORT,
				BSL432_UART_TXD_PIN, BSL432_UART_TXD_MODE);
	}

	/* Configure function pointers for current peripheral */
	BSL432_PI_sendByte = &BSL432_PI_sendByteUART;
	BSL432_PI_receiveByte = &BSL432_PI_receiveByteUART;
	BSL432_PI_sendData = &BSL432_PI_sendDataUART;
	BSL432_PI_receivePacket = &BSL432_PI_receivePacketUART;

	/* Configuring UART Module */
	switch (BSL432_UARTBaudRate) {
		case BSL432_UARTBaudRatesAuto:
			MAP_UART_initModule(BSL432_UART_MODULE, &BSL432_UARTConfig_auto);
			break;
		case BSL432_UARTBaudRates9600:
			MAP_UART_initModule(BSL432_UART_MODULE, &BSL432_UARTConfig_9600);
			break;
		case BSL432_UARTBaudRates19200:
			MAP_UART_initModule(BSL432_UART_MODULE, &BSL432_UARTConfig_19200);
			break;
		case BSL432_UARTBaudRates38400:
			MAP_UART_initModule(BSL432_UART_MODULE, &BSL432_UARTConfig_38400);
			break;
		case BSL432_UARTBaudRates57600:
			MAP_UART_initModule(BSL432_UART_MODULE, &BSL432_UARTConfig_57600);
			break;
		case BSL432_UARTBaudRates115200:
			MAP_UART_initModule(BSL432_UART_MODULE, &BSL432_UARTConfig_115200);
			break;
		default:
			MAP_UART_initModule(BSL432_UART_MODULE, &BSL432_UARTConfig_9600);
			break;
	}

	/* Enable UART module */
	MAP_UART_enableModule(BSL432_UART_MODULE);
}

/**
 * PI initialization routine for SPI.
 *
 * Configures the SPI module and send and receive functions.
 *
 * \return None
 */
void BSL432_PI_initSPI()
{
	const SPI_SlaveConfig BSL432_SPIConfig =
	{
		EUSCI_SPI_MSB_FIRST,
		EUSCI_SPI_PHASE_DATA_CHANGED_ONFIRST_CAPTURED_ON_NEXT,
		EUSCI_SPI_CLOCKPOLARITY_INACTIVITY_HIGH,
		EUSCI_SPI_4PIN_UCxSTE_ACTIVE_LOW
	};

	MAP_SPI_initSlave(BSL432_SPI_MODULE, &BSL432_SPIConfig);

	/* Enable UART module */
	MAP_SPI_enableModule(BSL432_SPI_MODULE);
}

/**
 * PI initialization routine for I2C.
 *
 * Configures the I2C module and send and receive functions.
 *
 * \return None
 */
void BSL432_PI_initI2C()
{
	MAP_I2C_initSlave(BSL432_I2C_MODULE, BSL432_I2C_SLAVE_ADDRESS,
			EUSCI_B_I2C_OWN_ADDRESS_OFFSET0, EUSCI_B_I2C_OWN_ADDRESS_ENABLE);

	/* Enable I2C module */
	MAP_I2C_enableModule(BSL432_I2C_MODULE);
}

/**
 * PI initialization routine for BSL timeout using Timer_A
 *
 * \return None
 */
void BSL432_PI_initTimeout()
{
	// Timer_A is sourced by ACLK (32768 Hz) with a devider of 10

	const Timer_A_UpModeConfig BSL432_TimeoutConfig =
	{
		TIMER_A_CLOCKSOURCE_ACLK,
		TIMER_A_CLOCKSOURCE_DIVIDER_10,
		BSL432_TIMEOUT,
		TIMER_A_TAIE_INTERRUPT_DISABLE,
		TIMER_A_CCIE_CCR0_INTERRUPT_DISABLE,
		TIMER_A_DO_CLEAR
	};

	MAP_Timer_A_stopTimer(TIMER_A0_MODULE);

	MAP_Timer_A_clearInterruptFlag(TIMER_A0_MODULE);

	MAP_Timer_A_configureUpMode(TIMER_A0_MODULE, &BSL432_TimeoutConfig);

	MAP_Timer_A_startCounter(TIMER_A0_MODULE, TIMER_A_UP_MODE);
}

/**
 * Processes PI commands (for UART only)
 *
 * returns None
 */
void BSL432_PI_interpretPICommand()
{
	uint8_t command = BSL432_ReceiveBuffer[0];
	if(command == BSL432_CHANGE_BAUD_RATE)
	{
		uint8_t rate = BSL432_ReceiveBuffer[1];
		if((rate < BSL432_UARTBaudRates9600) || (rate > BSL432_UARTBaudRates115200))
		{
			BSL432_PI_sendByte(BSL432_UNKNOWN_BAUD_RATE);
		}
		else
		{
			BSL432_PI_sendByte(BSL432_ACK);
			while(MAP_UART_queryStatusFlags(BSL432_UART_MODULE, EUSCI_A_UART_BUSY));
			BSL432_PI_initUART((BSL432_UARTBaudRates)rate);
		}
	}
}

/**
 * \return the maximum data buffer size for the current PI
 */
uint32_t BSL432_PI_getBufferSize()
{
	return (uint32_t)BSL432_MAX_BUFFER_SIZE;
}

/**
 * Verifies the data in the receive buffer against a given checksum
 */
uint32_t BSL432_PI_verifyData(uint32_t checksum)
{
	MAP_CRC32_setSeed(0xFFFF, CRC16_MODE);  // Initialize CRC

	uint32_t bufferIndex = 0;
	for(bufferIndex = 0; bufferIndex < BSL432_ReceiveBufferSize; bufferIndex++)
	{
		MAP_CRC32_set8BitDataReversed(BSL432_ReceiveBuffer[bufferIndex], CRC16_MODE);
	}
	return (MAP_CRC32_getResult(CRC16_MODE) == checksum);
}

/**
 * Reads and entire packet from the peripheral, verifies it,
 * and sends it to the core to be interpreted
 *
 * \return BSL event flags
 */
BSL432_eventFlags BSL432_PI_receivePacketUART()
{
	BSL432_PI_eventFlags = Flags_rxPacketOngoing;

	uint8_t dataByte = 0;
	uint32_t dataPointer = 0;
	uint32_t checksum = 0;

	while(BSL432_PI_eventFlags == Flags_rxPacketOngoing)
	{
		dataByte = BSL432_PI_receiveByte();  // get another byte from host
		if(dataPointer == 0)  // first byte is the size of the Core packet
		{
			if(dataByte != 0x80)  // first byte in packet should be 0x80
			{
				BSL432_PI_sendByte(BSL432_HEADER_INCORRECT);
				BSL432_PI_eventFlags = Flags_rxErrorRecoverable;
			}
			else
			{
				dataPointer++;
			}
		}
		else if (dataPointer == 1)
		{
			BSL432_ReceiveBufferSize = dataByte;
			dataPointer++;
		}
		else if (dataPointer == 2)
		{
			BSL432_ReceiveBufferSize |= (uint32_t)dataByte << 8;
			if(BSL432_ReceiveBufferSize == 0)
			{
				BSL432_PI_sendByte(BSL432_PACKET_SIZE_ZERO);
				BSL432_PI_eventFlags = Flags_rxErrorRecoverable;
			}
			if(BSL432_ReceiveBufferSize > BSL432_MAX_BUFFER_SIZE)  // For future devices that might need smaller packets
			{
				BSL432_PI_sendByte(BSL432_PACKET_SIZE_TOO_BIG);
				BSL432_PI_eventFlags = Flags_rxErrorRecoverable;
			}
			dataPointer++;
		}
		else if(dataPointer == (BSL432_ReceiveBufferSize + 3))
		{
			// if the pointer is pointing to the Checksum low data byte which resides
			// after 0x80, rSize, Core Command.
			checksum = dataByte;
			dataPointer++;
		}
		else if(dataPointer == (BSL432_ReceiveBufferSize + 4))
		{
			// if the pointer is pointing to the Checksum low data byte which resides
			// after 0x80, rSize, Core Command, CKL.
			checksum = checksum | dataByte<<8;
			if(BSL432_PI_verifyData(checksum))
			{
				if((BSL432_ReceiveBuffer[0] & 0xF0) == BSL432_PI_COMMAND_UPPER)
				{
					BSL432_PI_interpretPICommand();
					BSL432_PI_eventFlags = Flags_rxPacketOngoing;
					dataByte = 0;
					dataPointer = 0;
					checksum = 0;
				}
				else
				{
					BSL432_PI_sendByte(BSL432_ACK);
					BSL432_PI_eventFlags = Flags_data_received;
				}
			}
			else
			{
				BSL432_PI_sendByte(BSL432_CHECKSUM_INCORRECT);
				BSL432_PI_eventFlags = Flags_rxErrorRecoverable;
			}
		}
		else
		{
			BSL432_ReceiveBuffer[dataPointer-3] = dataByte;
			dataPointer++;
		}

	}
	return BSL432_PI_eventFlags;
}

/**
 * Sends a single byte via UART peripheral
 * Waits for the send buffer of the peripheral to be empty.
 *
 * \param data The byte to be transmitted.
 *
 * \return None
 */
void BSL432_PI_sendByteUART(uint8_t data)
{
	/* Wait until the TX buffer is empty, then send */
	while(!(UART_getInterruptStatus(BSL432_UART_MODULE, EUSCI_A_UART_TRANSMIT_INTERRUPT_FLAG)));
	MAP_UART_transmitData(BSL432_UART_MODULE, data);
}

/**
 * Receives a single byte from the UART peripheral
 *
 * \return The received byte.
 */
uint8_t BSL432_PI_receiveByteUART()
{
	return MAP_UART_receiveData(BSL432_UART_MODULE);
}

/**
 * Sends data in the data buffer via UART
 *
 * \param size The number of byte to send from the data buffer
 *
 * \return None
 */
void BSL432_PI_sendDataUART(uint32_t size)
{
	BSL432_PI_sendByteUART(0x80);  // BSL package header
	BSL432_PI_sendByteUART(size & 0xFF);  // BSL package size
	BSL432_PI_sendByteUART((size >> 8) & 0xFF);
	MAP_CRC32_setSeed(0xFFFF, CRC16_MODE);  // Initialize CRC
	uint32_t bufferIndex = 0;
	for(bufferIndex = 0; bufferIndex < size; bufferIndex++)
	{
		MAP_CRC32_set8BitDataReversed(BSL432_SendBuffer[bufferIndex], CRC16_MODE);
		MAP_UART_transmitData(BSL432_UART_MODULE, BSL432_SendBuffer[bufferIndex]);
	}
	BSL432_PI_sendByteUART(MAP_CRC32_getResult(CRC16_MODE) & 0xFF);  // BSL package CRC
	BSL432_PI_sendByteUART((MAP_CRC32_getResult(CRC16_MODE) >> 8) & 0xFF);
}

/**
 * Reads and entire packet from the SPI peripheral, verifies it,
 * and sends it to the core to be interpreted
 *
 * \return BSL event flags
 */
BSL432_eventFlags BSL432_PI_receivePacketSPI()
{
	BSL432_PI_eventFlags = Flags_rxPacketOngoing;

	uint8_t dataByte = 0;
	uint32_t dataPointer = 0;
	uint32_t checksum = 0;

	while(BSL432_PI_eventFlags == Flags_rxPacketOngoing)
	{
		dataByte = BSL432_PI_receiveByte();  // get another byte from host
		if(dataPointer == 0)  // first byte is the size of the Core packet
		{
			if(dataByte == 0xFF)
			{
				// Ignore reads when no command has been started.
			}
			else if(dataByte != 0x80)  // first byte in packet should be 0x80
			{
				BSL432_PI_sendByte(BSL432_HEADER_INCORRECT);
				BSL432_PI_eventFlags = Flags_rxErrorRecoverable;
			}
			else
			{
				dataPointer++;
			}
		}
		else if (dataPointer == 1)
		{
			BSL432_ReceiveBufferSize = dataByte;
			dataPointer++;
		}
		else if (dataPointer == 2)
		{
			BSL432_ReceiveBufferSize |= (uint32_t)dataByte << 8;
			if(BSL432_ReceiveBufferSize == 0)
			{
				BSL432_PI_sendByte(BSL432_PACKET_SIZE_ZERO);
				BSL432_PI_eventFlags = Flags_rxErrorRecoverable;
			}
			if(BSL432_ReceiveBufferSize > BSL432_MAX_BUFFER_SIZE)  // For future devices that might need smaller packets
			{
				BSL432_PI_sendByte(BSL432_PACKET_SIZE_TOO_BIG);
				BSL432_PI_eventFlags = Flags_rxErrorRecoverable;
			}
			dataPointer++;
		}
		else if(dataPointer == (BSL432_ReceiveBufferSize + 3))
		{
			// if the pointer is pointing to the Checksum low data byte which resides
			// after 0x80, rSize, Core Command.
			checksum = dataByte;
			dataPointer++;
		}
		else if(dataPointer == (BSL432_ReceiveBufferSize + 4))
		{
			// if the pointer is pointing to the Checksum low data byte which resides
			// after 0x80, rSize, Core Command, CKL.
			checksum = checksum | dataByte<<8;
			if(BSL432_PI_verifyData(checksum))
			{
				if((BSL432_ReceiveBuffer[0] & 0xF0) == BSL432_PI_COMMAND_UPPER)
				{
					BSL432_PI_interpretPICommand();
					BSL432_PI_eventFlags = Flags_rxPacketOngoing;
					dataByte = 0;
					dataPointer = 0;
					checksum = 0;
				}
				else
				{
					BSL432_PI_sendByte(BSL432_ACK);
					BSL432_PI_eventFlags = Flags_data_received;
				}
			}
			else
			{
				BSL432_PI_sendByte(BSL432_CHECKSUM_INCORRECT);
				BSL432_PI_eventFlags = Flags_rxErrorRecoverable;
			}
		}
		else
		{
			BSL432_ReceiveBuffer[dataPointer-3] = dataByte;
			dataPointer++;
		}

	}
	return BSL432_PI_eventFlags;
}

/**
 * Sends a single byte via SPI peripheral
 * Waits for the send buffer of the peripheral to be empty.
 *
 * \param data The byte to be transmitted.
 *
 * \return None
 */
void BSL432_PI_sendByteSPI(uint8_t data)
{
	/* Wait until the TX buffer is empty, then send */
	while(!(MAP_SPI_getInterruptStatus(BSL432_SPI_MODULE) & EUSCI_SPI_TRANSMIT_INTERRUPT));
	MAP_SPI_transmitData(BSL432_SPI_MODULE, data);
}

/**
 * Receives a single byte from the SPI peripheral
 *
 * \return The received byte.
 */
uint8_t BSL432_PI_receiveByteSPI()
{
	while(!(SPI_getInterruptStatus(BSL432_SPI_MODULE) & EUSCI_SPI_RECEIVE_INTERRUPT));
	return MAP_SPI_receiveData(BSL432_SPI_MODULE);
}

/**
 * Sends data in the data buffer via SPI
 *
 * This function sends the data in the global BSL432_SendBuffer.
 *
 * \param size Number of bytes to send.
 *
 * \return None
 */
void BSL432_PI_sendDataSPI(uint32_t size)
{
	BSL432_PI_sendByteSPI(0x80);  // BSL package header
	BSL432_PI_sendByteSPI(size & 0xFF);  // BSL package size
	BSL432_PI_sendByteSPI((size >> 8) & 0xFF);
	MAP_CRC32_setSeed(0xFFFF, CRC16_MODE);  // Initialize CRC
	uint32_t bufferIndex = 0;
	for(bufferIndex = 0; bufferIndex < size; bufferIndex++)
	{
		MAP_CRC32_set8BitDataReversed(BSL432_SendBuffer[bufferIndex], CRC16_MODE);
		BSL432_PI_sendByteSPI(BSL432_SendBuffer[bufferIndex]);
	}
	BSL432_PI_sendByteSPI(MAP_CRC32_getResult(CRC16_MODE) & 0xFF);  // BSL package CRC
	BSL432_PI_sendByteSPI((MAP_CRC32_getResult(CRC16_MODE) >> 8) & 0xFF);
}

/**
 * Handles the receiving and sending of bytes via I2C.
 *
 * \return BSL432_eventFlags
 */
BSL432_eventFlags BSL432_PI_receivePacketI2C()
{
	uint8_t RxValue = 0;

	BSL432_PI_eventFlags = Flags_rxPacketOngoing;

	while (BSL432_PI_eventFlags == Flags_rxPacketOngoing)
	{
		if(I2C_getInterruptStatus(BSL432_I2C_MODULE, EUSCI_B_I2C_START_INTERRUPT))
		{
			BSL432_PI_TxDataIndexI2C = 0;  // Start condition received
			BSL432_PI_RxDataIndexI2C = 0;
			I2C_clearInterruptFlag(BSL432_I2C_MODULE, EUSCI_B_I2C_START_INTERRUPT);
		}
		else if(I2C_getInterruptStatus(BSL432_I2C_MODULE, EUSCI_B_I2C_STOP_INTERRUPT))
		{
			// Stop condition received
			I2C_clearInterruptFlag(BSL432_I2C_MODULE, EUSCI_B_I2C_STOP_INTERRUPT);
		}
		else if(I2C_getInterruptStatus(BSL432_I2C_MODULE, EUSCI_B_I2C_RECEIVE_INTERRUPT0))
		{
			RxValue = MAP_I2C_slaveGetData(BSL432_I2C_MODULE);  // Data received
			BSL432_PI_receiveDataFromMasterI2C(RxValue);
		}
		else if(I2C_getInterruptStatus(BSL432_I2C_MODULE, EUSCI_B_I2C_TRANSMIT_INTERRUPT0))
		{
			if (BSL432_PI_TxDataIndexI2C < BSL432_SendBufferSize)
			{
				MAP_I2C_slavePutData(BSL432_I2C_MODULE, BSL432_RAM_TX_Buf[BSL432_PI_TxDataIndexI2C++]); // Send data
			}
		}
	}
	return BSL432_PI_eventFlags;
}

/**
 * Handles the receiving of BSL packets via I2C.
 *
 * \param dataByte Individual bytes of a BSL packet.
 *
 * \return None
 */
void BSL432_PI_receiveDataFromMasterI2C(uint8_t dataByte)
{
	volatile uint32_t checksum;  // Do not initialize
	if (BSL432_PI_RxDataIndexI2C == 0)  // First byte is the size of the Core packet
	{
		if (dataByte != 0x80)  // First byte in packet should be 0x80
		{
			BSL432_PI_sendByteI2C(BSL432_HEADER_INCORRECT);
			BSL432_PI_eventFlags = Flags_rxErrorRecoverable;
		}
		else
		{
			BSL432_PI_RxDataIndexI2C++;
		}
	}
	else if(BSL432_PI_RxDataIndexI2C == 1)  // First byte is the size of the Core packet
	{
		BSL432_ReceiveBufferSize = dataByte;
		BSL432_PI_RxDataIndexI2C++;
	}
	else if(BSL432_PI_RxDataIndexI2C == 2)
	{
		BSL432_ReceiveBufferSize |= (uint32_t)dataByte << 8;
		if(BSL432_ReceiveBufferSize == 0)
		{
			BSL432_PI_sendByteI2C(BSL432_PACKET_SIZE_ZERO);
			BSL432_PI_eventFlags = Flags_rxErrorRecoverable;
		}
		if(BSL432_ReceiveBufferSize > BSL432_MAX_BUFFER_SIZE)  // For future devices that might need smaller packets
		{
			BSL432_PI_sendByteI2C(BSL432_PACKET_SIZE_TOO_BIG);
			BSL432_PI_eventFlags = Flags_rxErrorRecoverable;
		}
		BSL432_PI_RxDataIndexI2C++;
	}
	else if(BSL432_PI_RxDataIndexI2C == (BSL432_ReceiveBufferSize + 3))
	{
		// If the pointer is pointing to the Checksum low data byte which resides
        // after 0x80, rSize, Core Command.
		checksum = dataByte;
		BSL432_PI_RxDataIndexI2C++;
	}
	else if(BSL432_PI_RxDataIndexI2C == (BSL432_ReceiveBufferSize + 4))
	{
		// If the pointer is pointing to the Checksum low data byte which resides
		// after 0x80, rSize, Core Command, CKL.
		checksum = checksum | dataByte << 8;
		if(BSL432_PI_verifyData(checksum))
		{
			BSL432_PI_sendByteI2C(BSL432_ACK);
			BSL432_PI_eventFlags = Flags_data_received;
		}
		else
		{
			BSL432_PI_sendByteI2C(BSL432_CHECKSUM_INCORRECT);
			BSL432_PI_eventFlags = Flags_rxErrorRecoverable;
		}
	}
	else
	{
		BSL432_RAM_RX_Buf[BSL432_PI_RxDataIndexI2C - 3] = dataByte;
		BSL432_PI_RxDataIndexI2C++;
	}
}

/**
 * Places a single byte to be sent into the transmit buffer
 *
 * \param data The byte to be transmitted.
 *
 * \return None
 */
void BSL432_PI_sendByteI2C(uint8_t data)
{
	BSL432_RAM_TX_Buf[0] = data; // Status Byte ACK/NACK
	BSL432_SendBufferSize = 1;
}

/**
 * Sends data in the data buffer via I2C
 *
 * This function sends the data in the global BSL432_SendBuffer.
 *
 * \param size Number of bytes to send.
 *
 * \return None
 */
void BSL432_PI_sendDataI2C(uint32_t size)
{
	uint32_t bufferIndex = 0;
	uint32_t rxStart = 0;  // Flag to check for start condition before transmiting
	uint32_t txPoint = 0;  // Use local variables to reduce code size, change to
	uint32_t txLen = BSL432_SendBufferSize;  // transmitPoint_I2C and transmitBufferLength_I2C if there is space

	/*
	 * Use transmitBufferLength_I2C when sending data to allow for either an
	 * acknowledgement for the first packet or a follow up packet (as used when
	 * requesting more than 259 bytes).
	 */

	BSL432_RAM_TX_Buf[txLen++] = 0x80;
	BSL432_RAM_TX_Buf[txLen++] = size & 0xFF;
	BSL432_RAM_TX_Buf[txLen++] = size >> 8 & 0xFF;
	MAP_CRC32_setSeed(0xFFFF, CRC16_MODE);  // Initialize CRC
	for(bufferIndex = 0; bufferIndex < size; bufferIndex++)
	{
		/*
		 * If transmitting multiple packets such as transmit data we must
		 * shift buffer down.
		 */
		MAP_CRC32_set8BitDataReversed(BSL432_SendBuffer[bufferIndex], CRC16_MODE);
		BSL432_RAM_TX_Buf[txLen++] = BSL432_SendBuffer[bufferIndex];
	}
	BSL432_RAM_TX_Buf[txLen++] = MAP_CRC32_getResult(CRC16_MODE) & 0xFF;
	BSL432_RAM_TX_Buf[txLen++] = (MAP_CRC32_getResult(CRC16_MODE) >> 8) & 0xFF;

	while(txPoint < txLen)
	{
		if(I2C_getInterruptStatus(BSL432_I2C_MODULE, EUSCI_B_I2C_START_INTERRUPT))
		{
			// Start condition received
			rxStart = 1;
			I2C_clearInterruptFlag(BSL432_I2C_MODULE, EUSCI_B_I2C_START_INTERRUPT);
		}
		else if(I2C_getInterruptStatus(BSL432_I2C_MODULE, EUSCI_B_I2C_STOP_INTERRUPT))
		{
			// Stop condition received
			I2C_clearInterruptFlag(BSL432_I2C_MODULE, EUSCI_B_I2C_STOP_INTERRUPT);
		}
		else if(I2C_getInterruptStatus(BSL432_I2C_MODULE, EUSCI_B_I2C_RECEIVE_INTERRUPT0))
		{
			// Data received
		}
		else if(I2C_getInterruptStatus(BSL432_I2C_MODULE, EUSCI_B_I2C_TRANSMIT_INTERRUPT0))
		{
			if(rxStart)
			{
				MAP_I2C_slavePutData(BSL432_I2C_MODULE, BSL432_RAM_TX_Buf[txPoint++]);  // Send data
			}
		}
	}
	BSL432_SendBufferSize = 0;
}
